/* eslint-disable @typescript-eslint/no-explicit-any */
import { AxiosError } from 'axios';
import SlackHook, { SlackMessage } from 'winston-slack-webhook-transport';

import { LOG_LEVEL } from '#/logger/logger.types';

import { AttachementField, Attachment, ErrorMessageWithAttachment } from './slack.transport.types';

type InfoData = Error | AxiosError | unknown | unknown[];

type Info = {
	context: string | undefined;
	message: string | undefined;
	level: keyof typeof LOG_LEVEL;
	data: InfoData | undefined;
	timestamp: string | undefined;
};

type GetMessageForSlack = {
	info: Info;
	nodeEnv: string;
	appName: string;
};

export const getMessageForSlack = ({ info, nodeEnv, appName }: GetMessageForSlack): false | SlackMessage => {
	if (typeof info !== 'object') return false;

	const errorObject = {
		context: info.context ?? undefined,
		message: info?.message ?? undefined,
		level: (info.level as string) ?? undefined,
		data: info.data ?? undefined,
		timestamp: (info.timestamp as string) ?? undefined,
	};

	const { data, context, message } = errorObject;
	let errorMessage = message;

	const attachments: Attachment[] = [];
	const fields: AttachementField[] = [];

	if (data !== undefined) {
		if (data instanceof Error || data instanceof AxiosError) {
			errorMessage = data.message;
			if (data.stack !== undefined) {
				const stackParts = data.stack.split('\n');

				stackParts.forEach((part) =>
					fields.push({
						title: '',
						value: part.trim(),
						short: false,
					})
				);
			}
		}
	}

	attachments.push({
		// green
		color: '#36a64f',
		fields,
		footer: `*${errorObject.timestamp} - ${nodeEnv}*`.toUpperCase(),
	});

	const slackMessage: SlackMessage = {
		attachments,
		blocks: [
			/**
			 * Header
			 */
			{
				type: 'header',
				text: {
					type: 'plain_text',
					text: `${errorObject.level.toUpperCase()}:exclamation:`,
					emoji: true,
				},
			},

			/**
			 * context
			 */
			{
				type: 'rich_text',
				elements: [
					{
						type: 'rich_text_section',
						elements: [
							{
								type: 'text',
								text: 'Error in: ',
							},
							{
								type: 'text',
								text: appName,
								style: {
									bold: true,
								},
							},
						],
					},
				],
			},
			{
				type: 'divider',
			},
			/**
			 * Context  Message
			 */
			{
				type: 'section',
				fields: [
					{
						type: 'mrkdwn',
						text: '*Message:*',
					},
					{
						type: 'mrkdwn',
						text: message,
					},
				],
			},
			{
				type: 'divider',
			},
			/**
			 * Context
			 */
			{
				type: 'section',
				fields: [
					{
						type: 'mrkdwn',
						text: '*Context:*',
					},
					{
						type: 'mrkdwn',
						text: context,
					},
				],
			},
			{
				type: 'divider',
			},

			/**
			 * Error Stack or Response
			 */
			{
				type: 'section',
				fields: [
					{
						type: 'mrkdwn',
						text: '*Error:*',
					},
					{
						type: 'mrkdwn',
						text: errorMessage,
					},
				],
			},
		],
	};
	return slackMessage;
};

type SlackCreateParams = {
	nodeEnv: string;
	webhookUrl: string;
	appName: string;
};
export default class SlackTransport {
	public static create({ appName, webhookUrl, nodeEnv }: SlackCreateParams) {
		return new SlackHook({
			level: LOG_LEVEL.Error || LOG_LEVEL.Fatal || LOG_LEVEL.Emergency,
			webhookUrl,
			formatter: (info: any) => getMessageForSlack({ appName, info, nodeEnv }),
		});
	}
}
